@@ -3,7 +3,7 @@
  * Child class of abstract VinagreConnection, specific to VNC protocol
  * This file is part of vinagre
  *
- * Copyright (C) 2009 - Jonh Wendell <wendell@bani.com.br>
+ * Copyright (C) 2009-2010 - Jonh Wendell <wendell@bani.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,8 @@
 #include <glib/gi18n.h>
 #include <stdlib.h>
 #include <vinagre/vinagre-utils.h>
+#include <vinagre/vinagre-cache-prefs.h>
+
 #include "vinagre-vnc-connection.h"
 
 struct _VinagreVncConnectionPrivate
@@ -29,10 +31,13 @@ struct _VinagreVncConnectionPrivate
   gchar    *desktop_name;
   gboolean view_only;
   gboolean scaling;
+  gboolean keep_ratio;
   gint     shared;
   gint     fd;
   gint     depth_profile;
   gboolean lossy_encoding;
+  gchar    *ssh_tunnel_host;
+  GSocket  *socket;
 };
 
 enum
@@ -41,10 +46,13 @@ enum
   PROP_DESKTOP_NAME,
   PROP_VIEW_ONLY,
   PROP_SCALING,
+  PROP_KEEP_RATIO,
   PROP_SHARED,
   PROP_FD,
   PROP_DEPTH_PROFILE,
-  PROP_LOSSY_ENCODING
+  PROP_LOSSY_ENCODING,
+  PROP_SSH_TUNNEL_HOST,
+  PROP_SOCKET
 };
 
 #define VINAGRE_VNC_CONNECTION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), VINAGRE_TYPE_VNC_CONNECTION, VinagreVncConnectionPrivate))
@@ -58,10 +66,13 @@ vinagre_vnc_connection_init (VinagreVncConnection *conn)
   conn->priv->desktop_name = NULL;
   conn->priv->view_only = FALSE;
   conn->priv->scaling = FALSE;
+  conn->priv->keep_ratio = FALSE;
   conn->priv->shared = -1;
   conn->priv->fd = 0;
   conn->priv->depth_profile = 0;
   conn->priv->lossy_encoding = FALSE;
+  conn->priv->ssh_tunnel_host = NULL;
+  conn->priv->socket = NULL;
 }
 
 static void
@@ -76,10 +87,25 @@ vinagre_vnc_connection_finalize (GObject *object)
   VinagreVncConnection *conn = VINAGRE_VNC_CONNECTION (object);
 
   g_free (conn->priv->desktop_name);
+  g_free (conn->priv->ssh_tunnel_host);
 
   G_OBJECT_CLASS (vinagre_vnc_connection_parent_class)->finalize (object);
 }
 
+static void
+vinagre_vnc_connection_dispose (GObject *object)
+{
+  VinagreVncConnection *conn = VINAGRE_VNC_CONNECTION (object);
+
+  if (conn->priv->socket)
+    {
+      g_object_unref (conn->priv->socket);
+      conn->priv->socket = NULL;
+    }
+
+  G_OBJECT_CLASS (vinagre_vnc_connection_parent_class)->dispose (object);
+}
+
 static void
 vinagre_vnc_connection_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
@@ -103,6 +129,10 @@ vinagre_vnc_connection_set_property (GObject *object, guint prop_id, const GValu
 	vinagre_vnc_connection_set_scaling (conn, g_value_get_boolean (value));
 	break;
 
+      case PROP_KEEP_RATIO:
+	vinagre_vnc_connection_set_keep_ratio (conn, g_value_get_boolean (value));
+	break;
+
       case PROP_SHARED:
 	vinagre_vnc_connection_set_shared (conn, g_value_get_int (value));
 	break;
@@ -119,6 +149,14 @@ vinagre_vnc_connection_set_property (GObject *object, guint prop_id, const GValu
 	vinagre_vnc_connection_set_lossy_encoding (conn, g_value_get_boolean (value));
 	break;
 
+      case PROP_SSH_TUNNEL_HOST:
+	vinagre_vnc_connection_set_ssh_tunnel_host (conn, g_value_get_string (value));
+	break;
+
+      case PROP_SOCKET:
+	vinagre_vnc_connection_set_socket (conn, g_value_get_object (value));
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -148,12 +186,16 @@ vinagre_vnc_connection_get_property (GObject *object, guint prop_id, GValue *val
 	g_value_set_boolean (value, conn->priv->scaling);
 	break;
 
+      case PROP_KEEP_RATIO:
+	g_value_set_boolean (value, conn->priv->keep_ratio);
+	break;
+
       case PROP_SHARED:
 	g_value_set_int (value, conn->priv->shared);
 	break;
 
       case PROP_FD:
-	g_value_set_int (value, conn->priv->fd);
+	g_value_set_int (value, vinagre_vnc_connection_get_fd (conn));
 	break;
 
       case PROP_DEPTH_PROFILE:
@@ -164,6 +206,14 @@ vinagre_vnc_connection_get_property (GObject *object, guint prop_id, GValue *val
 	g_value_set_boolean (value, conn->priv->lossy_encoding);
 	break;
 
+      case PROP_SSH_TUNNEL_HOST:
+	g_value_set_string (value, conn->priv->ssh_tunnel_host);
+	break;
+
+      case PROP_SOCKET:
+	g_value_set_object (value, conn->priv->socket);
+	break;
+
       default:
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	break;
@@ -178,8 +228,12 @@ vnc_fill_writer (VinagreConnection *conn, xmlTextWriter *writer)
 
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"view_only", "%d", vnc_conn->priv->view_only);
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"scaling", "%d", vnc_conn->priv->scaling);
+  xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"keep_ratio", "%d", vnc_conn->priv->keep_ratio);
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"depth_profile", "%d", vnc_conn->priv->depth_profile);
   xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"lossy_encoding", "%d", vnc_conn->priv->lossy_encoding);
+
+  if (vnc_conn->priv->ssh_tunnel_host && *vnc_conn->priv->ssh_tunnel_host)
+    xmlTextWriterWriteFormatElement (writer, (const xmlChar *)"ssh_tunnel_host", "%s", vnc_conn->priv->ssh_tunnel_host);
 }
 
 static void
@@ -204,6 +258,10 @@ vnc_parse_item (VinagreConnection *conn, xmlNode *root)
 	  if (!scaling_command_line)
 	    vinagre_vnc_connection_set_scaling (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
 	}
+      else if (!xmlStrcmp(curr->name, (const xmlChar *)"keep_ratio"))
+	{
+	  vinagre_vnc_connection_set_keep_ratio (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
+	}
       else if (!xmlStrcmp(curr->name, (const xmlChar *)"depth_profile"))
 	{
 	  vinagre_vnc_connection_set_depth_profile (vnc_conn, atoi((const char *)s_value));
@@ -212,6 +270,10 @@ vnc_parse_item (VinagreConnection *conn, xmlNode *root)
 	{
 	  vinagre_vnc_connection_set_lossy_encoding (vnc_conn, vinagre_utils_parse_boolean ((const gchar *)s_value));
 	}
+      else if (!xmlStrcmp(curr->name, (const xmlChar *)"ssh_tunnel_host"))
+	{
+	  vinagre_vnc_connection_set_ssh_tunnel_host (vnc_conn, (const gchar *)s_value);
+	}
 
       xmlFree (s_value);
     }
@@ -250,29 +312,41 @@ vnc_fill_conn_from_file (VinagreConnection *conn, GKeyFile *file)
     if (shared == 0 || shared == 1)
       vinagre_vnc_connection_set_shared (VINAGRE_VNC_CONNECTION (conn), shared);
     else
+      /* Translators: 'shared' here is a VNC protocol specific flag. You can translate it, but I think it's better to let it untranslated */
       g_message (_("Bad value for 'shared' flag: %d. It is supposed to be 0 or 1. Ignoring it."), shared);
 }
 
 static void
 vnc_parse_options_widget (VinagreConnection *conn, GtkWidget *widget)
 {
-  GtkWidget *view_only, *scaling, *depth_combo, *lossy;
+  GtkWidget *view_only, *scaling, *depth_combo, *lossy, *ssh_host, *ratio;
 
   view_only = g_object_get_data (G_OBJECT (widget), "view_only");
   scaling = g_object_get_data (G_OBJECT (widget), "scaling");
+  ratio = g_object_get_data (G_OBJECT (widget), "ratio");
   depth_combo = g_object_get_data (G_OBJECT (widget), "depth_combo");
   lossy = g_object_get_data (G_OBJECT (widget), "lossy");
-  if (!view_only || !scaling || !depth_combo || !lossy)
+  ssh_host = g_object_get_data (G_OBJECT (widget), "ssh_host");
+  if (!view_only || !scaling || !depth_combo || !lossy || !ssh_host || !ratio)
     {
       g_warning ("Wrong widget passed to vnc_parse_options_widget()");
       return;
     }
 
+  vinagre_cache_prefs_set_boolean ("vnc-connection", "view-only", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view_only)));
+  vinagre_cache_prefs_set_boolean ("vnc-connection", "scaling", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (scaling)));
+  vinagre_cache_prefs_set_boolean ("vnc-connection", "keep-ratio", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ratio)));
+  vinagre_cache_prefs_set_integer ("vnc-connection", "depth-profile", gtk_combo_box_get_active (GTK_COMBO_BOX (depth_combo)));
+  vinagre_cache_prefs_set_boolean ("vnc-connection", "lossy-encoding", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lossy)));
+  vinagre_cache_prefs_set_string  ("vnc-connection", "ssh-tunnel-host", gtk_entry_get_text (GTK_ENTRY (ssh_host)));
+
   g_object_set (conn,
 		"view-only", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (view_only)),
 		"scaling", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (scaling)),
+		"keep-ratio", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ratio)),
 		"depth-profile", gtk_combo_box_get_active (GTK_COMBO_BOX (depth_combo)),
 		"lossy-encoding", gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (lossy)),
+		"ssh-tunnel-host", gtk_entry_get_text (GTK_ENTRY (ssh_host)),
 		NULL);
 }
 
@@ -285,6 +359,7 @@ vinagre_vnc_connection_class_init (VinagreVncConnectionClass *klass)
   g_type_class_add_private (klass, sizeof (VinagreVncConnectionPrivate));
 
   object_class->finalize = vinagre_vnc_connection_finalize;
+  object_class->dispose = vinagre_vnc_connection_dispose;
   object_class->set_property = vinagre_vnc_connection_set_property;
   object_class->get_property = vinagre_vnc_connection_get_property;
   object_class->constructed  = vinagre_vnc_connection_constructed;
@@ -330,6 +405,18 @@ vinagre_vnc_connection_class_init (VinagreVncConnectionClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_KEEP_RATIO,
+                                   g_param_spec_boolean ("keep-ratio",
+                                                        "Keep Ratio",
+	                                                "Whether to keep the aspect ratio when using scaling",
+                                                        FALSE,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property (object_class,
                                    PROP_SHARED,
                                    g_param_spec_int ("shared",
@@ -384,6 +471,30 @@ vinagre_vnc_connection_class_init (VinagreVncConnectionClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property (object_class,
+                                   PROP_SSH_TUNNEL_HOST,
+                                   g_param_spec_string ("ssh-tunnel-host",
+                                                        "SSH Tunnel Host",
+	                                                "hostname used to create the SSH tunnel",
+                                                        NULL,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+
+  g_object_class_install_property (object_class,
+                                   PROP_SOCKET,
+                                   g_param_spec_object ("socket",
+                                                        "Socket",
+	                                                "A GSocket for this connection",
+	                                                G_TYPE_SOCKET,
+	                                                G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT |
+                                                        G_PARAM_STATIC_NICK |
+                                                        G_PARAM_STATIC_NAME |
+                                                        G_PARAM_STATIC_BLURB));
+
 }
 
 VinagreConnection *
@@ -458,6 +569,22 @@ vinagre_vnc_connection_get_scaling (VinagreVncConnection *conn)
   return conn->priv->scaling;
 }
 
+void
+vinagre_vnc_connection_set_keep_ratio (VinagreVncConnection *conn,
+				       gboolean value)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+
+  conn->priv->keep_ratio = value;
+}
+gboolean
+vinagre_vnc_connection_get_keep_ratio (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), FALSE);
+
+  return conn->priv->keep_ratio;
+}
+
 void
 vinagre_vnc_connection_set_fd (VinagreVncConnection *conn,
 			       gint                 value)
@@ -472,7 +599,27 @@ vinagre_vnc_connection_get_fd (VinagreVncConnection *conn)
 {
   g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), 0);
 
-  return conn->priv->fd;
+  if (conn->priv->socket)
+    return g_socket_get_fd (conn->priv->socket);
+  else
+    return conn->priv->fd;
+}
+
+void
+vinagre_vnc_connection_set_socket (VinagreVncConnection *conn,
+				   GSocket              *socket)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+
+  if (socket)
+    conn->priv->socket = g_object_ref (socket);
+}
+GSocket *
+vinagre_vnc_connection_get_socket (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), 0);
+
+  return conn->priv->socket;
 }
 
 void
@@ -508,4 +655,21 @@ vinagre_vnc_connection_get_lossy_encoding (VinagreVncConnection *conn)
   return conn->priv->lossy_encoding;
 }
 
+void
+vinagre_vnc_connection_set_ssh_tunnel_host (VinagreVncConnection *conn,
+					    const gchar *host)
+{
+  g_return_if_fail (VINAGRE_IS_VNC_CONNECTION (conn));
+
+  g_free (conn->priv->ssh_tunnel_host);
+  conn->priv->ssh_tunnel_host = g_strdup (host);
+}
+const gchar *
+vinagre_vnc_connection_get_ssh_tunnel_host (VinagreVncConnection *conn)
+{
+  g_return_val_if_fail (VINAGRE_IS_VNC_CONNECTION (conn), NULL);
+
+  return conn->priv->ssh_tunnel_host;
+}
+
 /* vim: set ts=8: */
