@@ -23,7 +23,6 @@
 
 #include <gtk/gtk.h>
 #include <glib.h>
-#include "vinagre-plugin.h"
 
 /* useful macro */
 #define GBOOLEAN_TO_POINTER(i) ((gpointer) ((i) ? 2 : 1))
@@ -46,27 +45,16 @@ void		vinagre_utils_toggle_widget_visible	(GtkWidget *widget);
 
 const gchar	*vinagre_utils_get_ui_filename		(void);
 const gchar	*vinagre_utils_get_ui_xml_filename	(void);
-GtkBuilder	*vinagre_utils_get_builder		(VinagrePlugin *plugin, const gchar *filename);
+GtkBuilder	*vinagre_utils_get_builder		(const gchar *filename);
 
 gchar		*vinagre_utils_escape_underscores	(const gchar *text,
 							 gssize      length);
 
-void		vinagre_utils_handle_debug		(void);
-
-guint		vinagre_utils_get_current_workspace	(GdkScreen *screen);
-guint		vinagre_utils_get_window_workspace	(GtkWindow *gtkwindow);
-void		vinagre_utils_get_current_viewport	(GdkScreen    *screen,
-							 gint         *x,
-							 gint         *y);
-
 void		vinagre_utils_help_contents		(GtkWindow *window, const gchar *section);
 void		vinagre_utils_help_about		(GtkWindow *window);
 
 gboolean	vinagre_utils_parse_boolean		(const gchar* value);
 
-GtkWidget      *vinagre_gtk_button_new_with_stock_icon   (const gchar *label,
-                                                         const gchar *stock_id);
-
 gboolean	vinagre_utils_ask_question		(GtkWindow  *parent,
 							 const char *message,
 							 char       **choices,
